const fs = require('fs')
const {execSync} = require('child_process')
const {writeFileSync} = require('fs')
const path = require('path')
const getNodeModulesPath = require('./getNodeModulesPath')
const { Logger } = require('./Logger');
const {plugins: pluginsFromCommonRepo} = require('../CommonConfig/PluginsList/version');

function installPluginIfNotInstalled(){
const nodeModulesPathOfProject = getNodeModulesPath()
let packageJsonFilePath = path.join(nodeModulesPathOfProject,'package.json')
let uninstalledPlugins = []
let pluginsToBeAddedInDevDependencies = []
let packageJsonContent = require(packageJsonFilePath)
let pluginsIndevDependencies = packageJsonContent.devDependencies
let arePluginsInstalledSuccessfully = false
let isPackageJsonChanged = false
let isPackageJsonChangesRevertedSuccessfully = false

let pluginNamesOfDevDependencyPlugins = Object.keys(pluginsIndevDependencies)

for(let pluginFromCommonRepo in pluginsFromCommonRepo){
    if(! pluginNamesOfDevDependencyPlugins.includes(pluginFromCommonRepo)){
        let pluginToBeInstalled = `${pluginFromCommonRepo}@${pluginsFromCommonRepo[pluginFromCommonRepo]}`
        uninstalledPlugins.push(pluginToBeInstalled)
    }
    else if(pluginNamesOfDevDependencyPlugins.includes(pluginFromCommonRepo)){
        if(pluginsFromCommonRepo[pluginFromCommonRepo] !== pluginsIndevDependencies[pluginFromCommonRepo]){
            let pluginToBeInstalled = `${pluginFromCommonRepo}@${pluginsFromCommonRepo[pluginFromCommonRepo]}`
            uninstalledPlugins.push(pluginToBeInstalled)
        }
    }
}
if(uninstalledPlugins.length > 0){
    try{
        process.chdir(nodeModulesPathOfProject)
        let installCommand = `npm install --save-dev ${uninstalledPlugins.join(' ')}`
        try{
            execSync(installCommand,{stdio:'inherit'})
            arePluginsInstalledSuccessfully = true
            Logger.log(Logger.SUCCESS_TYPE,'Plugins are installed successfully')
            if(arePluginsInstalledSuccessfully){
                try{
                    process.chdir(nodeModulesPathOfProject)
                    try{
                        let gitDiffOutput = execSync(`git diff --name-only`).toString().trim()
                        isPackageJsonChanged = gitDiffOutput.includes('package.json') ? true : false
                        if(isPackageJsonChanged){
                            try{
                                let packageJsonChangesRevertCommand = `git checkout "${packageJsonFilePath}"`
                                execSync(packageJsonChangesRevertCommand,{stdio:'inherit'})
                                isPackageJsonChangesRevertedSuccessfully = true
                                Logger.log(Logger.SUCCESS_TYPE,'Changes in package.json reverted successfully')
                                
                                if(isPackageJsonChangesRevertedSuccessfully){
                                    uninstalledPlugins.map(uninstalledPlugin => {
                                        if(uninstalledPlugin.startsWith('@')){
                                            let indexOfAtCharacter = uninstalledPlugin.indexOf('@')
                                            let indexOfSecondOccurenceOfAtCharacter = uninstalledPlugin.indexOf('@',indexOfAtCharacter + 1)
                                            let unInstalledPluginName = uninstalledPlugin.slice(0,indexOfSecondOccurenceOfAtCharacter)
                                            let unInstalledPluginVersion = uninstalledPlugin.slice(indexOfSecondOccurenceOfAtCharacter + 1)
                                            pluginsToBeAddedInDevDependencies[unInstalledPluginName] = unInstalledPluginVersion
                                        }
                                        else{
                                            pluginsToBeAddedInDevDependencies[uninstalledPlugin.split('@')[0]] = uninstalledPlugin.split('@')[1]
                                        }
                                    })
                                    let updatedPluginsIndevDependencies = {
                                        ...pluginsIndevDependencies,
                                        ...pluginsToBeAddedInDevDependencies
                                    }
                                    packageJsonContent.devDependencies = updatedPluginsIndevDependencies
                                    
                                    let modifiedPackageJson = {
                                        ...packageJsonContent,
                                    }
                                    try{
                                        writeFileSync(`${packageJsonFilePath}`,JSON.stringify(modifiedPackageJson,null,2))
                                        Logger.log(Logger.SUCCESS_TYPE,'Newly installed plugins successfully added to package.json')
                                    }
                                    catch(error){
                                        Logger.log(Logger.INFO_TYPE,error)
                                        Logger.log(Logger.FAILURE_TYPE,'Unable to append installed plugins to package.json')
                                    }
                                }
                            }
                            catch(err){
                                isPackageJsonChangesRevertedSuccessfully = false
                                Logger.log(Logger.INFO_TYPE,err)
                                Logger.log(Logger.FAILURE_TYPE,'Unable to revert the changes in package.json file')
                            }
                        }
                        else{
                            Logger.log(Logger.SUCCESS_TYPE,'package.json not changed')
                        }
                    }
                    catch(err){
                        Logger.log(Logger.INFO_TYPE,err)
                        Logger.log(Logger.FAILURE_TYPE,'Unable to execute git diff command while checking if package.json is changed')
                    }
                }
                catch(err){
                    Logger.log(Logger.INFO_TYPE,err)
                    Logger.log(Logger.FAILURE_TYPE,'Unable to navigate to node_modules path during reverting changes in package.json')
                }
            }
        }
        catch(err){
            arePluginsInstalledSuccessfully = false
            Logger.log(Logger.INFO_TYPE,err)
            Logger.log(Logger.FAILURE_TYPE,'Unable to install the plugins')
        }
    }
    catch(error){
        Logger.log(Logger.FAILURE_TYPE,'Unable to navigate to node_modules path of project while trying to install plugins')
    }
}
else if(uninstalledPlugins.length === 0){
    Logger.log(Logger.INFO_TYPE,'Plugins are installed already')
    arePluginsInstalledSuccessfully = true
}
}


module.exports = {
    installPluginIfNotInstalled
}